@@ -74,16 +74,22 @@ def user_loc_string_to_value(axis_tag, user_loc):
     >>> user_loc_string_to_value('wdth', 'Clearly Not From Glyphs UI')
     """
     if axis_tag == "wght":
-        try:
-            value = _nospace_lookup(WEIGHT_CODES, user_loc)
-        except KeyError:
-            return None
+        if isinstance(user_loc, int):
+            value = user_loc
+        else:
+            try:
+                value = _nospace_lookup(WEIGHT_CODES, user_loc)
+            except KeyError:
+                return None
         return class_to_value("wght", value)
     elif axis_tag == "wdth":
-        try:
-            value = _nospace_lookup(WIDTH_CODES, user_loc)
-        except KeyError:
-            return None
+        if isinstance(user_loc, int):
+            value = user_loc
+        else:
+            try:
+                value = _nospace_lookup(WIDTH_CODES, user_loc)
+            except KeyError:
+                return None
         return class_to_value("wdth", value)
 
     # Currently this function should only be called with a width or weight
@@ -263,7 +269,7 @@ def font_uses_axis_locations(font):
     # resort to using instances or other old tricks to get the mapping.
     # https://github.com/googlefonts/glyphsLib/issues/409
     # https://github.com/googlefonts/glyphsLib/issues/411
-    return font.customParameters["Axes"] and all(
+    return font.axes and all(
         master.customParameters["Axis Location"] for master in font.masters
     )
 
@@ -272,16 +278,18 @@ def to_glyphs_axes(self):
     axes_parameter = []
     for axis in self.designspace.axes:
         if axis.tag == "wght":
-            axes_parameter.append({"Name": axis.name or "Weight", "Tag": "wght"})
+            axes_parameter.append(
+                classes.GSAxis(name=axis.name or "Weight", tag="wght")
+            )
         elif axis.tag == "wdth":
-            axes_parameter.append({"Name": axis.name or "Width", "Tag": "wdth"})
+            axes_parameter.append(classes.GSAxis(name=axis.name or "Width", tag="wdth"))
         else:
-            axes_parameter.append({"Name": axis.name, "Tag": axis.tag})
+            axes_parameter.append(classes.GSAxis(name=axis.name, tag=axis.tag))
 
     if axes_parameter and not _is_subset_of_default_axes(axes_parameter):
-        self.font.customParameters["Axes"] = axes_parameter
+        self.font.axes = axes_parameter
 
-    if any(a.map for a in self.designspace.axes):
+    if any(_has_meaningful_map(a, self.designspace) for a in self.designspace.axes):
         mapping = {
             axis.tag: {str(k): v for k, v in axis.map} for axis in self.designspace.axes
         }
@@ -317,11 +325,11 @@ class AxisDefinition:
         axis it could be the thickness of a stem, for the width a percentage
         of extension with respect to the normal width.
         """
-        return getattr(glyphs_master_or_instance, self.design_loc_key)
+        return glyphs_master_or_instance._get_axis_value(self.design_loc_key)
 
     def set_design_loc(self, master_or_instance, value):
         """Set the design location of a Glyphs master or instance."""
-        setattr(master_or_instance, self.design_loc_key, value)
+        master_or_instance._set_axis_value(self.design_loc_key, value)
 
     def get_user_loc(self, master_or_instance):
         """Get the user location of a Glyphs master or instance.
@@ -484,14 +492,7 @@ class AxisDefinitionFactory:
         return AxisDefinition(tag, name, design_loc_key, 0.0, None, None, 0.0)
 
     def _design_loc_key(self):
-        if self.axis_index == 0:
-            return "weightValue"
-        elif self.axis_index == 1:
-            return "widthValue"
-        elif self.axis_index == 2:
-            return "customValue"
-        else:
-            return "customValue%d" % (self.axis_index - 2)
+        return self.axis_index
 
 
 defaults_factory = AxisDefinitionFactory()
@@ -516,15 +517,41 @@ def _is_subset_of_default_axes(axes_parameter):
     if len(axes_parameter) > 3:
         return False
     for axis, axis_def in zip(axes_parameter, DEFAULT_AXES_DEFS):
-        if set(axis.keys()) != {"Name", "Tag"}:
+        if axis.name != axis_def.name:
             return False
-        if axis["Name"] != axis_def.name:
-            return False
-        if axis["Tag"] != axis_def.tag:
+        if axis.axisTag != axis_def.tag:
             return False
     return True
 
 
+def _has_meaningful_map(axis, designspace):
+    if not axis.map:
+        return False
+    for k, v in axis.map:
+        if k != v:
+            return True
+    # We have an identity map. We could elide it, but...
+    # sometimes we use an identity map to force a particular
+    # range even though the sources don't fill that range.
+    min_axis = None
+    max_axis = None
+    for source in designspace.sources:
+        loc = source.location.get(axis.name)
+        if loc is None:
+            continue
+        if min_axis is None:
+            min_axis = loc
+        else:
+            min_axis = min(loc, min_axis)
+        if max_axis is None:
+            max_axis = loc
+        else:
+            max_axis = max(loc, max_axis)
+    if min_axis != axis.map[0][0] or max_axis != axis.map[-1][0]:
+        return True
+    return False
+
+
 def get_regular_master(font):
     """Find the "regular" master among the GSFontMasters.
 
