@@ -23,13 +23,16 @@ from typing import Any, Dict
 
 from fontTools import designspaceLib
 
-from glyphsLib import classes, util
-from .constants import PUBLIC_PREFIX, FONT_CUSTOM_PARAM_PREFIX, GLYPHLIB_PREFIX
+from glyphsLib import classes, glyphdata, util
+from .constants import (
+    PUBLIC_PREFIX,
+    FONT_CUSTOM_PARAM_PREFIX,
+    GLYPHLIB_PREFIX,
+)
 from .axes import WEIGHT_AXIS_DEF, WIDTH_AXIS_DEF, find_base_style, class_to_value
 
 GLYPH_ORDER_KEY = PUBLIC_PREFIX + "glyphOrder"
 
-BRACKET_LAYER_RE = re.compile(r".*(?P<first_bracket>[\[\]])\s*(?P<value>\d+)\s*\].*")
 BRACKET_GLYPH_TEMPLATE = "{glyph_name}.{rev}BRACKET.{location}"
 REVERSE_BRACKET_LABEL = "REV_"
 BRACKET_GLYPH_RE = re.compile(
@@ -63,12 +66,14 @@ class UFOBuilder(_LoggerMixin):
         designspace_module=designspaceLib,
         family_name=None,
         instance_dir=None,
-        propagate_anchors=True,
+        propagate_anchors=None,
         use_designspace=False,
         minimize_glyphs_diffs=False,
         generate_GDEF=True,
         store_editor_state=True,
         write_skipexportglyphs=False,
+        minimal=False,
+        glyph_data=None,
     ):
         """Create a builder that goes from Glyphs to UFO + designspace.
 
@@ -83,7 +88,9 @@ class UFOBuilder(_LoggerMixin):
                        only instances with this name will be returned.
         instance_dir -- if provided, instance UFOs will be located in this
                         directory, according to their Designspace filenames.
-        propagate_anchors -- set to False to prevent anchor propagation
+        propagate_anchors -- set to True or False to explicitly control anchor
+                             propagation, the default is to check for
+                             "Propagate Anchors" custom parameter.
         use_designspace -- set to True to make optimal use of the designspace:
                            data that is common to all ufos will go there.
         minimize_glyphs_diffs -- set to True to store extra info in UFOs
@@ -97,6 +104,7 @@ class UFOBuilder(_LoggerMixin):
                                          into the UFOs' and Designspace's lib instead
                                          of the glyph level lib key
                                          "com.schriftgestaltung.Glyphs.Export".
+        glyph_data -- A list of GlyphData.
         """
         self.font = font
 
@@ -107,18 +115,32 @@ class UFOBuilder(_LoggerMixin):
 
         self.designspace_module = designspace_module
         self.instance_dir = instance_dir
-        self.propagate_anchors = propagate_anchors
         self.use_designspace = use_designspace
         self.minimize_glyphs_diffs = minimize_glyphs_diffs
         self.generate_GDEF = generate_GDEF
         self.store_editor_state = store_editor_state
         self.bracket_layers = []
         self.write_skipexportglyphs = write_skipexportglyphs
+        self.minimal = minimal
+
+        if propagate_anchors is None:
+            propagate_anchors = font.customParameters["Propagate Anchors"]
+            propagate_anchors = bool(propagate_anchors is None or propagate_anchors)
+        self.propagate_anchors = propagate_anchors
 
         # The set of (SourceDescriptor + UFO)s that will be built,
         # indexed by master ID, the same order as masters in the source GSFont.
         self._sources = OrderedDict()
 
+        # Map Glyphs layer IDs to UFO layer names.
+        self._layer_map = {}
+
+        # List of exploded color palette layers when building minimal UFOs.
+        self._color_palette_layers = []
+
+        # List of color layers when building minimal UFOs.
+        self._color_layers = []
+
         # A cache for mappings of layer IDs to mappings of glyph names to Glyphs layers,
         # for passing into pens as glyph sets.
         self._glyph_sets: Dict[str, Dict[str, classes.GSLayer]] = {}
@@ -168,6 +190,17 @@ class UFOBuilder(_LoggerMixin):
             # instances with matching 'familyName' custom parameter
             self._do_filter_instances_by_family = True
 
+        if glyph_data:
+            from io import BytesIO
+
+            glyphdata_files = []
+            for path in glyph_data:
+                with open(path, "rb") as fp:
+                    glyphdata_files.append(BytesIO(fp.read()))
+            self.glyphdata = glyphdata.GlyphData.from_files(*glyphdata_files)
+        else:
+            self.glyphdata = None
+
     def _is_vertical(self):
         master_ids = {m.id for m in self.font.masters}
         for glyph in self.font.glyphs:
@@ -180,13 +213,59 @@ class UFOBuilder(_LoggerMixin):
 
     @property
     def masters(self):
-        """Get an iterator over master UFOs that match the given family_name.
-        """
+        """Get an iterator over master UFOs that match the given family_name."""
         if self._sources:
             for source in self._sources.values():
                 yield source.font
             return
 
+        # TODO(jamesgk) maybe create one font at a time to reduce memory usage
+        # TODO: (jany) in the future, return a lazy iterator that builds UFOs
+        #     on demand.
+        self.to_ufo_font_attributes(self.family_name)
+
+        self.to_ufo_layers()
+
+        for master_id, source in self._sources.items():
+            ufo = source.font
+            master = self.font.masters[master_id]
+            if self.propagate_anchors:
+                self.to_ufo_propagate_font_anchors(ufo)
+            for layer in list(ufo.layers):
+                self.to_ufo_layer_lib(master, ufo, layer)
+
+            # Color layer mapping is stored using layer IDs, we now rewrite it
+            # to use the final UFO layer names.
+            self.to_ufo_color_layer_names(master, ufo)
+
+            # to_ufo_custom_params may apply "Replace Features" or "Replace Prefix"
+            # parameters so it requires UFOs have their features set first; at the
+            # same time, to generate a GDEF table we first need to have defined the
+            # glyphOrder, exported the glyphs and propagated anchors from components.
+            self.to_ufo_master_features(ufo, master)
+            self.to_ufo_custom_params(ufo, master)
+
+            self.to_ufo_color_layers(ufo, master)
+
+        if self.write_skipexportglyphs:
+            # Sanitize skip list and write it to both Designspace- and UFO-level lib
+            # keys. The latter is unnecessary when using e.g. the ufo2ft.compile*FromDS`
+            # functions, but the data may take a different path. Writing it everywhere
+            # can save on surprises/logic in other software.
+            skip_export_glyphs = self._designspace.lib.get("public.skipExportGlyphs")
+            if skip_export_glyphs is not None:
+                skip_export_glyphs = sorted(set(skip_export_glyphs))
+                self._designspace.lib["public.skipExportGlyphs"] = skip_export_glyphs
+                for source in self._sources.values():
+                    source.font.lib["public.skipExportGlyphs"] = skip_export_glyphs
+
+        self.to_ufo_groups()
+        self.to_ufo_kerning()
+
+        for source in self._sources.values():
+            yield source.font
+
+    def to_ufo_layers(self):
         # Store set of actually existing master (layer) ids. This helps with
         # catching dangling layer data that Glyphs may ignore, e.g. when
         # copying glyphs from other fonts with, naturally, different master
@@ -197,11 +276,6 @@ class UFOBuilder(_LoggerMixin):
         # stores background data from "associated layers"
         supplementary_layer_data = []
 
-        # TODO(jamesgk) maybe create one font at a time to reduce memory usage
-        # TODO: (jany) in the future, return a lazy iterator that builds UFOs
-        #     on demand.
-        self.to_ufo_font_attributes(self.family_name)
-
         # Generate the main (master) layers first.
         for glyph in self.font.glyphs:
             for layer in glyph.layers.values():
@@ -247,49 +321,22 @@ class UFOBuilder(_LoggerMixin):
             # set to non-export (in which case makes no sense to have Designspace rules
             # referencing non existent glyphs).
             if (
-                BRACKET_LAYER_RE.match(layer.name)
+                layer._is_bracket_layer()
                 and glyph.export
                 and ".background" not in layer.name
             ):
                 self.bracket_layers.append(layer)
+            elif (
+                self.minimal
+                and layer.layerId not in master_layer_ids
+                and not layer._is_brace_layer()
+            ):
+                continue
             else:
                 ufo_layer = self.to_ufo_layer(glyph, layer)
                 ufo_glyph = ufo_layer.newGlyph(glyph.name)
                 self.to_ufo_glyph(ufo_glyph, layer, layer.parent)
 
-        for master_id, source in self._sources.items():
-            ufo = source.font
-            if self.propagate_anchors:
-                self.to_ufo_propagate_font_anchors(ufo)
-            for layer in ufo.layers:
-                self.to_ufo_layer_lib(layer)
-
-            master = self.font.masters[master_id]
-            # to_ufo_custom_params may apply "Replace Features" or "Replace Prefix"
-            # parameters so it requires UFOs have their features set first; at the
-            # same time, to generate a GDEF table we first need to have defined the
-            # glyphOrder, exported the glyphs and propagated anchors from components.
-            self.to_ufo_master_features(ufo, master)
-            self.to_ufo_custom_params(ufo, master)
-
-        if self.write_skipexportglyphs:
-            # Sanitize skip list and write it to both Designspace- and UFO-level lib
-            # keys. The latter is unnecessary when using e.g. the ufo2ft.compile*FromDS`
-            # functions, but the data may take a different path. Writing it everywhere
-            # can save on surprises/logic in other software.
-            skip_export_glyphs = self._designspace.lib.get("public.skipExportGlyphs")
-            if skip_export_glyphs is not None:
-                skip_export_glyphs = sorted(set(skip_export_glyphs))
-                self._designspace.lib["public.skipExportGlyphs"] = skip_export_glyphs
-                for source in self._sources.values():
-                    source.font.lib["public.skipExportGlyphs"] = skip_export_glyphs
-
-        self.to_ufo_groups()
-        self.to_ufo_kerning()
-
-        for source in self._sources.values():
-            yield source.font
-
     @property
     def designspace(self):
         """Get a designspace Document instance that links the masters together
@@ -380,27 +427,13 @@ class UFOBuilder(_LoggerMixin):
         # can go through the layers by location and copy them to free-standing glyphs
         bracket_layer_map = defaultdict(partial(defaultdict, list))
         for layer in self.bracket_layers:
+            bracket_axis_id, bracket_min, bracket_max = self.validate_bracket_info(
+                layer, bracket_axis, bracket_axis_min, bracket_axis_max
+            )
+            if bracket_min is None and bracket_max is None:
+                continue
             glyph_name = layer.parent.name
-
-            m = BRACKET_LAYER_RE.match(layer.name)
-            assert m
-
-            reverse = m.group("first_bracket") == "]"
-            bracket_crossover = int(m.group("value"))
-
-            if not bracket_axis_min <= bracket_crossover <= bracket_axis_max:
-                raise ValueError(
-                    "Glyph {glyph_name}: Bracket layer {layer_name} must be within the "
-                    "design space bounds of the {bracket_axis_name} axis: minimum "
-                    "{bracket_axis_minimum}, maximum {bracket_axis_maximum}.".format(
-                        glyph_name=glyph_name,
-                        layer_name=layer.name,
-                        bracket_axis_name=bracket_axis.name,
-                        bracket_axis_minimum=bracket_axis_min,
-                        bracket_axis_maximum=bracket_axis_max,
-                    )
-                )
-            bracket_layer_map[glyph_name][(bracket_crossover, reverse)].append(layer)
+            bracket_layer_map[glyph_name][(bracket_min, bracket_max)].append(layer)
 
         # Sort crossovers into rule buckets, one for regular bracket layers (in which
         # the location represents the min value) and one for 'reverse' bracket layers
@@ -410,11 +443,11 @@ class UFOBuilder(_LoggerMixin):
         for glyph_name, glyph_bracket_layers in sorted(bracket_layer_map.items()):
             min_crossovers = set()
             max_crossovers = set()
-            for location, reverse in glyph_bracket_layers.keys():
-                if reverse:
-                    max_crossovers.add(location)
-                else:
-                    min_crossovers.add(location)
+            for bracket_min, bracket_max in glyph_bracket_layers.keys():
+                if bracket_min is not None:
+                    min_crossovers.add(bracket_min)
+                elif bracket_max is not None:
+                    max_crossovers.add(bracket_max)
             # reverse and non-reverse bracket layers with overlapping ranges are
             # tricky to implement as DS rules. They are relatively unlikely, and
             # can usually be rewritten so that they do not overlap. For laziness/
@@ -430,15 +463,17 @@ class UFOBuilder(_LoggerMixin):
                     glyph_name,
                     ", ".join("]{}] > [{}]".format(*values) for values in invalid_locs),
                 )
-            for crossover_min, crossover_max in util.pairwise(
-                [bracket_axis_min] + sorted(max_crossovers)
-            ):
+            max_crossovers = list(sorted(max_crossovers))
+            if bracket_axis_min not in max_crossovers:
+                max_crossovers = [bracket_axis_min] + max_crossovers
+            for crossover_min, crossover_max in util.pairwise(max_crossovers):
                 max_rule_bucket[(int(crossover_min), int(crossover_max))].append(
                     glyph_name
                 )
-            for crossover_min, crossover_max in util.pairwise(
-                sorted(min_crossovers) + [bracket_axis_max]
-            ):
+            min_crossovers = list(sorted(min_crossovers))
+            if bracket_axis_max not in min_crossovers:
+                min_crossovers = min_crossovers + [bracket_axis_max]
+            for crossover_min, crossover_max in util.pairwise(min_crossovers):
                 min_rule_bucket[(int(crossover_min), int(crossover_max))].append(
                     glyph_name
                 )
@@ -465,6 +500,45 @@ class UFOBuilder(_LoggerMixin):
         if self.generate_GDEF:
             self.regenerate_gdef()
 
+    def validate_bracket_info(
+        self, layer, bracket_axis, bracket_axis_min, bracket_axis_max
+    ):
+        bracket_axis_id, bracket_min, bracket_max = layer._bracket_info()
+
+        if bracket_axis_id != 0:
+            raise ValueError("For now, bracket layers can only apply to the first axis")
+
+        # Convert [500<wght<(max)] to [500<wght], etc.
+        if bracket_min == bracket_axis_min:
+            bracket_min = None
+        if bracket_max == bracket_axis_max:
+            bracket_max = None
+
+        if bracket_min is not None and bracket_max is not None:
+            raise ValueError("Alternate rules with min and max range not yet supported")
+
+        glyph_name = layer.parent.name
+
+        if (
+            bracket_min is not None
+            and not bracket_axis_min <= bracket_min <= bracket_axis_max
+        ) or (
+            bracket_max is not None
+            and not bracket_axis_min <= bracket_max <= bracket_axis_max
+        ):
+            raise ValueError(
+                "Glyph {glyph_name}: Bracket layer {layer_name} must be within the "
+                "design space bounds of the {bracket_axis_name} axis: minimum "
+                "{bracket_axis_minimum}, maximum {bracket_axis_maximum}.".format(
+                    glyph_name=glyph_name,
+                    layer_name=layer.name,
+                    bracket_axis_name=bracket_axis.name,
+                    bracket_axis_minimum=bracket_axis_min,
+                    bracket_axis_maximum=bracket_axis_max,
+                )
+            )
+        return bracket_axis_id, bracket_min, bracket_max
+
     def _copy_bracket_layers_to_ufo_glyphs(self, bracket_layer_map):
         font = self.font
         master_ids = {m.id for m in font.masters}
@@ -479,7 +553,10 @@ class UFOBuilder(_LoggerMixin):
 
         for glyph_name, glyph_bracket_layers in bracket_layer_map.items():
             glyph = font.glyphs[glyph_name]
-            for (location, reverse), bracket_layers in glyph_bracket_layers.items():
+            for (
+                (bracket_min, bracket_max),
+                bracket_layers,
+            ) in glyph_bracket_layers.items():
 
                 for missing_master_layer_id in master_ids.difference(
                     bl.associatedMasterId for bl in bracket_layers
@@ -488,10 +565,24 @@ class UFOBuilder(_LoggerMixin):
                     bracket_layers.append(master_layer)
                     implicit_bracket_layers.add(id(master_layer))
 
+                if bracket_max is None:
+                    reverse = False
+                    location = bracket_min
+                else:
+                    reverse = True
+                    location = bracket_max
+
                 bracket_glyphs.add(_bracket_glyph_name(glyph_name, reverse, location))
 
         for glyph_name, glyph_bracket_layers in bracket_layer_map.items():
-            for (location, reverse), layers in glyph_bracket_layers.items():
+            for (bracket_min, bracket_max), layers in glyph_bracket_layers.items():
+                if bracket_max is None:
+                    reverse = False
+                    location = bracket_min
+                else:
+                    reverse = True
+                    location = bracket_max
+
                 for layer in layers:
                     layer_id = layer.associatedMasterId or layer.layerId
                     ufo_font = self._sources[layer_id].font
@@ -523,6 +614,7 @@ class UFOBuilder(_LoggerMixin):
     from .axes import to_designspace_axes
     from .background_image import to_ufo_background_image
     from .blue_values import to_ufo_blue_values
+    from .color_layers import to_ufo_color_layers
     from .common import to_ufo_time
     from .components import to_ufo_components, to_ufo_smart_component_axes
     from .custom_params import to_ufo_custom_params
@@ -533,7 +625,7 @@ class UFOBuilder(_LoggerMixin):
     from .hints import to_ufo_hints
     from .instances import to_designspace_instances
     from .kerning import to_ufo_kerning
-    from .layers import to_ufo_layer, to_ufo_background_layer
+    from .layers import to_ufo_layer, to_ufo_background_layer, to_ufo_color_layer_names
     from .masters import to_ufo_master_attributes
     from .names import to_ufo_names
     from .paths import to_ufo_paths
@@ -670,7 +762,6 @@ class GlyphsBuilder(_LoggerMixin):
             self.skip_export_glyphs = set()
 
         self._font = None
-        """The GSFont that will be built."""
 
     @property
     def font(self):
@@ -736,7 +827,7 @@ class GlyphsBuilder(_LoggerMixin):
                         del layer[glyph_name]
 
             for layer in _sorted_backgrounds_last(source.font.layers):
-                self.to_glyphs_layer_lib(layer)
+                self.to_glyphs_layer_lib(layer, master)
                 for glyph in layer:
                     self.to_glyphs_glyph(glyph, layer, master)
 
