@@ -18,8 +18,10 @@ import re
 
 from glyphsLib.util import bin_to_int_list, int_list_to_bin
 from .filters import parse_glyphs_filter
+from .common import to_ufo_color
 from .constants import (
     GLYPHS_PREFIX,
+    UFO2FT_COLOR_PALETTES_KEY,
     UFO2FT_FILTERS_KEY,
     UFO2FT_USE_PROD_NAMES_KEY,
     CODEPAGE_RANGES,
@@ -146,6 +148,18 @@ class GlyphsObjectProxy:
         opposed to a master or instance."""
         return hasattr(self._owner, "glyphs")
 
+    def has_properties(self):
+        if self.is_font():
+            return self._owner.format_version > 2
+        else:
+            return hasattr(self._owner, "properties")
+
+    def get_property(self, key):
+        for prop in self._owner.properties:
+            if key == prop.key:
+                return prop.defaultValue
+        return None
+
 
 class UFOProxy:
     """Record access to the UFO's lib custom parameters"""
@@ -198,6 +212,7 @@ class ParamHandler(AbstractParamHandler):
         ufo_name=None,
         glyphs_long_name=None,
         glyphs_multivalued=False,
+        glyphs3_property=None,
         ufo_prefix=CUSTOM_PARAM_PREFIX,
         ufo_info=True,
         ufo_default=None,
@@ -207,6 +222,7 @@ class ParamHandler(AbstractParamHandler):
         self.glyphs_name = glyphs_name
         self.glyphs_long_name = glyphs_long_name
         self.glyphs_multivalued = glyphs_multivalued
+        self.glyphs3_property = glyphs3_property
         # By default, they have the same name in both
         self.ufo_name = ufo_name or glyphs_name
         self.ufo_prefix = ufo_prefix
@@ -234,6 +250,9 @@ class ParamHandler(AbstractParamHandler):
         self._write_to_ufo(glyphs, ufo, ufo_value)
 
     def _read_from_glyphs(self, glyphs):
+        # Is it now a property?
+        if self.glyphs3_property and glyphs.has_properties():
+            return glyphs.get_property(self.glyphs3_property)
         # Try both the prefixed (long) name and the short name
         if self.glyphs_multivalued:
             getter = glyphs.get_custom_values
@@ -248,6 +267,11 @@ class ParamHandler(AbstractParamHandler):
         return None
 
     def _write_to_glyphs(self, glyphs, value):
+        # We currently convert UFO to Glyphs2 files.
+        # If we ever export Glyphs3 by default, we need a similar test
+        # here to the one in _read_from_glyphs to determine whether a
+        # value should be placed in the new properties top-level key.
+
         # Never write the prefixed (long) name?
         # FIXME: (jany) maybe should rather preserve the naming choice of user
         if self.glyphs_multivalued:
@@ -290,13 +314,8 @@ GLYPHS_UFO_CUSTOM_PARAMS = (
     ("hheaDescender", "openTypeHheaDescender"),
     ("hheaLineGap", "openTypeHheaLineGap"),
     ("compatibleFullName", "openTypeNameCompatibleFullName"),
-    ("description", "openTypeNameDescription"),
-    ("license", "openTypeNameLicense"),
-    ("licenseURL", "openTypeNameLicenseURL"),
     ("preferredFamilyName", "openTypeNamePreferredFamilyName"),
     ("preferredSubfamilyName", "openTypeNamePreferredSubfamilyName"),
-    ("sampleText", "openTypeNameSampleText"),
-    ("WWSFamilyName", "openTypeNameWWSFamilyName"),
     ("WWSSubfamilyName", "openTypeNameWWSSubfamilyName"),
     # OS/2 parameters
     ("panose", "openTypeOS2Panose"),
@@ -305,7 +324,6 @@ GLYPHS_UFO_CUSTOM_PARAMS = (
     ("typoDescender", "openTypeOS2TypoDescender"),
     ("typoLineGap", "openTypeOS2TypoLineGap"),
     ("unicodeRanges", "openTypeOS2UnicodeRanges"),
-    ("vendorID", "openTypeOS2VendorID"),
     ("strikeoutSize", "openTypeOS2StrikeoutSize"),
     ("strikeoutPosition", "openTypeOS2StrikeoutPosition"),
     # OS/2 Subscript parameters
@@ -338,6 +356,29 @@ GLYPHS_UFO_CUSTOM_PARAMS = (
 for glyphs_name, ufo_name in GLYPHS_UFO_CUSTOM_PARAMS:
     register(ParamHandler(glyphs_name, ufo_name, glyphs_long_name=ufo_name))
 
+GLYPHS_UFO_CUSTOM_PARAMS_GLYPHS3_PROPERTIES = (
+    ("license", "openTypeNameLicense", "licenses"),
+    ("licenseURL", "openTypeNameLicenseURL", "licenseURL"),
+    ("trademark", "trademark", "trademarks"),
+    ("description", "openTypeNameDescription", "descriptions"),
+    ("sampleText", "openTypeNameSampleText", "sampleTexts"),
+    ("postscriptFontName", "postscriptFontName", "postscriptFontName"),
+    ("postscriptFullName", "postscriptFullName", "postscriptFullName"),
+    ("WWSFamilyName", "openTypeNameWWSFamilyName", "WWSFamilyName"),
+    ("vendorID", "openTypeOS2VendorID", "vendorID"),
+    ("versionString", "openTypeNameVersion", "versionString"),
+)
+
+for glyphs_name, ufo_name, property_name in GLYPHS_UFO_CUSTOM_PARAMS_GLYPHS3_PROPERTIES:
+    register(
+        ParamHandler(
+            glyphs_name,
+            ufo_name,
+            glyphs_long_name=ufo_name,
+            glyphs3_property=property_name,
+        )
+    )
+
 # TODO: (jany) for all the following fields, check that they are stored in a
 # meaningful Glyphs customParameter. Maybe they have short names?
 GLYPHS_UFO_CUSTOM_PARAMS_NO_SHORT_NAME = (
@@ -352,8 +393,6 @@ GLYPHS_UFO_CUSTOM_PARAMS_NO_SHORT_NAME = (
     "openTypeNameVersion",
     "openTypeNameUniqueID",
     "openTypeOS2FamilyClass",
-    "postscriptFontName",
-    "postscriptFullName",
     "postscriptSlantAngle",
     "postscriptUniqueID",
     # Should this be handled in `blue_values.py`?
@@ -368,7 +407,6 @@ GLYPHS_UFO_CUSTOM_PARAMS_NO_SHORT_NAME = (
     "postscriptWindowsCharacterSet",
     "macintoshFONDFamilyID",
     "macintoshFONDName",
-    "trademark",
     "styleMapFamilyName",
     "styleMapStyleName",
 )
@@ -376,10 +414,6 @@ for name in GLYPHS_UFO_CUSTOM_PARAMS_NO_SHORT_NAME:
     register(ParamHandler(name))
 
 
-# TODO: (jany) handle dynamic version number replacement
-register(ParamHandler("versionString", "openTypeNameVersion"))
-
-
 class EmptyListDefaultParamHandler(ParamHandler):
     def to_glyphs(self, glyphs, ufo):
         ufo_value = self._read_from_ufo(glyphs, ufo)
@@ -485,6 +519,41 @@ register(
     )
 )
 
+register(
+    ParamHandler(
+        glyphs_name="gasp Table",
+        ufo_name="openTypeGaspRangeRecords",
+        value_to_ufo=to_ufo_gasp_table,
+        value_to_glyphs=to_glyphs_gasp_table,
+    )
+)
+
+
+def to_ufo_color_palettes(value):
+    return [[to_ufo_color(color) for color in palette] for palette in value]
+
+
+def _to_glyphs_color(color):
+    if color[0] == color[1] == color[2]:
+        color = [color[0], color[3]]
+    return ",".join(str(round(v * 255)) for v in color)
+
+
+def to_glyphs_color_palettes(value):
+    return [[_to_glyphs_color(color) for color in palette] for palette in value]
+
+
+register(
+    ParamHandler(
+        glyphs_name="Color Palettes",
+        ufo_name=UFO2FT_COLOR_PALETTES_KEY,
+        ufo_info=False,
+        ufo_prefix="",
+        value_to_ufo=to_ufo_color_palettes,
+        value_to_glyphs=to_glyphs_color_palettes,
+    )
+)
+
 # TODO: (jany) look at
 # https://forum.glyphsapp.com/t/name-table-entry-win-id4/3811/10
 # Use Name Table Entry for the next param
@@ -760,7 +829,7 @@ register(ReplaceFeatureParamHandler())
 
 
 class ReencodeGlyphsParamHandler(AbstractParamHandler):
-    """ The "Reencode Glyphs" custom parameter contains a list of
+    """The "Reencode Glyphs" custom parameter contains a list of
     'glyphname=unicodevalue' strings: e.g., ["smiley=E100", "logo=E101"].
     It only applies to specific instance (not to master or globally) and is
     meant to assign Unicode values to glyphs with the specied name at export
@@ -869,8 +938,7 @@ DEFAULT_PARAMETERS = (
 
 
 def _set_default_params(ufo):
-    """ Set Glyphs.app's default parameters when different from ufo2ft ones.
-    """
+    """Set Glyphs.app's default parameters when different from ufo2ft ones."""
     for _, ufo_name, default_value in DEFAULT_PARAMETERS:
         if getattr(ufo.info, ufo_name) is None:
             if isinstance(default_value, list):
@@ -881,7 +949,7 @@ def _set_default_params(ufo):
 
 
 def _unset_default_params(glyphs):
-    """ Unset Glyphs.app's parameters that have default values.
+    """Unset Glyphs.app's parameters that have default values.
     FIXME: (jany) maybe this should be taken care of in the writer? and/or
         classes should have better default values?
     """
