@@ -20,6 +20,7 @@ from io import StringIO
 from fontTools.feaLib import ast, parser
 
 from .constants import GLYPHLIB_PREFIX
+from .tokens import TokenExpander, PassThruExpander
 
 
 ANONYMOUS_FEATURE_PREFIX_NAME = "<anonymous>"
@@ -42,10 +43,116 @@ def to_ufo_master_features(self, ufo, master):
             ufo,
             generate_GDEF=self.generate_GDEF,
             skip_export_glyphs=skip_export_glyphs,
+            master=master,
         )
 
 
-def _to_ufo_features(font, ufo=None, generate_GDEF=False, skip_export_glyphs=None):
+_LANGUAGE_MAPPING = {
+    "dflt": None,
+    "AFK": 0x0436,
+    "ARA": 0x0C01,
+    "ASM": 0x044D,
+    "AZE": 0x042C,
+    "BEL": 0x0423,
+    "BEN": 0x0845,
+    "BGR": 0x0402,
+    "BRE": 0x047E,
+    "CAT": 0x0403,
+    "CSY": 0x0405,
+    "DAN": 0x0406,
+    "DEU": 0x0407,
+    "ELL": 0x0408,
+    "ENG": 0x0409,
+    "ESP": 0x0C0A,
+    "ETI": 0x0425,
+    "EUQ": 0x042D,
+    "FIN": 0x040B,
+    "FLE": 0x0813,
+    "FOS": 0x0438,
+    "FRA": 0x040C,
+    "FRI": 0x0462,
+    "GRN": 0x046F,
+    "GUJ": 0x0447,
+    "HAU": 0x0468,
+    "HIN": 0x0439,
+    "HRV": 0x041A,
+    "HUN": 0x040E,
+    "HVE": 0x042B,
+    "IRI": 0x083C,
+    "ISL": 0x040F,
+    "ITA": 0x0410,
+    "ITA": 0x0410,
+    "IWR": 0x040D,
+    "JPN": 0x0411,
+    "KAN": 0x044B,
+    "KAT": 0x0437,
+    "KAZ": 0x043F,
+    "KHM": 0x0453,
+    "KOK": 0x0457,
+    "LAO": 0x0454,
+    "LSB": 0x082E,
+    "LTH": 0x0427,
+    "LVI": 0x0426,
+    "MAR": 0x044E,
+    "MKD": 0x042F,
+    "MLR": 0x044C,
+    "MLY": 0x043E,
+    "MNG": 0x0352,
+    "MTS": 0x043A,
+    "NEP": 0x0461,
+    "NLD": 0x0413,
+    "NOB": 0x0414,
+    "ORI": 0x0448,
+    "PAN": 0x0446,
+    "PAS": 0x0463,
+    "PLK": 0x0415,
+    "PTG": 0x0816,
+    "PTG-BR": 0x0416,
+    "RMS": 0x0417,
+    "ROM": 0x0418,
+    "RUS": 0x0419,
+    "SAN": 0x044F,
+    "SKY": 0x041B,
+    "SLV": 0x0424,
+    "SQI": 0x041C,
+    "SRB": 0x081A,
+    "SVE": 0x041D,
+    "TAM": 0x0449,
+    "TAT": 0x0444,
+    "TEL": 0x044A,
+    "THA": 0x041E,
+    "TIB": 0x0451,
+    "TRK": 0x041F,
+    "UKR": 0x0422,
+    "URD": 0x0420,
+    "USB": 0x042E,
+    "UYG": 0x0480,
+    "UZB": 0x0443,
+    "VIT": 0x042A,
+    "WEL": 0x0452,
+    "ZHH": 0x0C04,
+    "ZHS": 0x0804,
+    "ZHT": 0x0404,
+}
+
+_REVERSE_LANGUAGE_MAPPING = {v: k for v, k in _LANGUAGE_MAPPING.items()}
+
+
+def _to_name_langID(language):
+    if language not in _LANGUAGE_MAPPING:
+        raise ValueError(f"Unknown name language: {language}")
+    return _LANGUAGE_MAPPING[language]
+
+
+def _to_glyphs_language(langID):
+    if langID not in _REVERSE_LANGUAGE_MAPPING:
+        raise ValueError(f"Unknown name langID: {langID}")
+    return _REVERSE_LANGUAGE_MAPPING[langID]
+
+
+def _to_ufo_features(
+    font, ufo=None, generate_GDEF=False, skip_export_glyphs=None, master=None
+):
     """Convert GSFont features, including prefixes and classes, to UFO.
 
     Optionally, build a GDEF table definiton, excluding 'skip_export_glyphs'.
@@ -55,16 +162,22 @@ def _to_ufo_features(font, ufo=None, generate_GDEF=False, skip_export_glyphs=Non
         ufo: Optional[defcon.Font]
         generate_GDEF: bool
         skip_export_glyphs: Optional[List[str]]
+        master: Optional[GSFontMaster]
 
     Returns: str
     """
+    if not master:
+        expander = PassThruExpander()
+    else:
+        expander = TokenExpander(font, master)
+
     prefixes = []
     for prefix in font.featurePrefixes:
         strings = []
         if prefix.name != ANONYMOUS_FEATURE_PREFIX_NAME:
             strings.append("# Prefix: %s\n" % prefix.name)
         strings.append(autostr(prefix.automatic))
-        strings.append(prefix.code)
+        strings.append(expander.expand(prefix.code))
         prefixes.append("".join(strings))
 
     prefix_str = "\n\n".join(prefixes)
@@ -74,17 +187,51 @@ def _to_ufo_features(font, ufo=None, generate_GDEF=False, skip_export_glyphs=Non
         prefix = "@" if not class_.name.startswith("@") else ""
         name = prefix + class_.name
         class_defs.append(
-            "{}{} = [ {} ];".format(autostr(class_.automatic), name, class_.code)
+            "{}{} = [ {}\n];".format(
+                autostr(class_.automatic), name, expander.expand(class_.code)
+            )
         )
     class_str = "\n\n".join(class_defs)
 
     feature_defs = []
     for feature in font.features:
-        code = feature.code
+        code = expander.expand(feature.code)
         lines = ["feature %s {" % feature.name]
-        if feature.notes:
+        notes = feature.notes
+        feature_names = None
+        if font.format_version == 2 and notes:
+            m = re.search("(featureNames {.+};)", notes, flags=re.DOTALL)
+            if m:
+                name = m.groups()[0]
+                # Remove the name from the note
+                notes = notes.replace(name, "").strip()
+                feature_names = name.splitlines()
+            else:
+                m = re.search(r"^(Name: (.+))", notes)
+                if m:
+                    line, name = m.groups()
+                    # Remove the name from the note
+                    notes = notes.replace(line, "").strip()
+                    # Replace special chars backslash and doublequote for AFDKO syntax
+                    name = name.replace("\\", r"\005c").replace('"', r"\0022")
+                    feature_names = ["featureNames {", f'  name "{name}";', "};"]
+        elif font.format_version == 3 and feature.labels:
+            feature_names = []
+            feature_names.append("featureNames {")
+            for label in feature.labels:
+                langID = _to_name_langID(label["language"])
+                name = label["value"]
+                name = name.replace("\\", r"\005c").replace('"', r"\0022")
+                if langID is None:
+                    feature_names.append(f'  name "{name}";')
+                else:
+                    feature_names.append(f'  name 3 1 0x{langID:X} "{name}";')
+            feature_names.append("};")
+        if notes:
             lines.append("# notes:")
-            lines.extend("# " + line for line in feature.notes.splitlines())
+            lines.extend("# " + line for line in notes.splitlines())
+        if feature_names:
+            lines.extend(feature_names)
         if feature.automatic:
             lines.append("# automatic")
         if feature.disabled:
@@ -158,7 +305,9 @@ def _build_gdef(ufo, skipExportGlyphs=None):
 
         # First check glyph.lib for category/subCategory overrides. Otherwise,
         # use global values from GlyphData.
-        glyphinfo = glyphdata.get_glyph(glyph.name)
+        glyphinfo = glyphdata.get_glyph(
+            glyph.name, unicodes=[f"{c:04X}" for c in glyph.unicodes]
+        )
         category = glyph.lib.get(category_key) or glyphinfo.category
         subCategory = glyph.lib.get(subCategory_key) or glyphinfo.subCategory
 
@@ -225,7 +374,7 @@ def _replace_block(kind, tag, repl, features):
     if not repl.endswith("\n"):
         repl += "\n"
     return re.sub(
-        fr"(?<=^{kind} {tag} {{\n)(.*?)(?=^}} {tag};$)",
+        rf"(?<=^{kind} {tag} {{\n)(.*?)(?=^}} {tag};$)",
         repl,
         features,
         count=1,
@@ -595,21 +744,30 @@ class FeatureFileProcessor:
             if st.glyphs.original:
                 for glyph in st.glyphs.original:
                     try:
-                        # Class name (GlyphClassName object)
-                        elements.append("@" + glyph.glyphclass.name)
+                        # Class name (MarkClassName object)
+                        elements.append("@" + glyph.markClass.name)
                     except AttributeError:
                         try:
-                            # Class name (GlyphClassDefinition object)
-                            # FIXME: (jany) why not always the same type?
-                            elements.append("@" + glyph.name)
+                            # Class name (GlyphClassName object)
+                            elements.append("@" + glyph.glyphclass.name)
                         except AttributeError:
-                            # Glyph name
-                            elements.append(glyph)
+                            try:
+                                # Class name (GlyphClassDefinition object)
+                                # FIXME: (jany) why not always the same type?
+                                elements.append("@" + glyph.name)
+                            except AttributeError:
+                                # Glyph name
+                                elements.append(glyph)
             else:
                 elements = st.glyphSet()
         except AttributeError:
             # Single class
-            elements.append("@" + st.glyphs.glyphclass.name)
+            try:
+                # Class name (MarkClassName object)
+                elements.append("@" + st.glyphs.markClass.name)
+            except AttributeError:
+                # Class name (GlyphClassName object)
+                elements.append("@" + st.glyphs.glyphclass.name)
         glyph_class.code = " ".join(elements)
         glyph_class.automatic = bool(automatic)
         self._font.classes.append(glyph_class)
@@ -629,6 +787,46 @@ class FeatureFileProcessor:
         feature = self.glyphs_module.GSFeature()
         feature.name = st.name
         feature.automatic = bool(automatic)
+        if feature.automatic:
+            # See if there is a feature names block in the code that should be
+            # written to the notes.
+            for i, statement in enumerate(contents):
+                if (
+                    isinstance(statement, ast.NestedBlock)
+                    and statement.block_name == "featureNames"
+                ):
+                    feature_names = contents[i]
+                    if feature_names.statements:
+                        # If there is only one name has default platformID,
+                        # platEncID and langID, write it using the simple
+                        # syntax. Otherwise write out the full featureNames
+                        # statement.
+                        if self._font.format_version == 2:
+                            name = feature_names.statements[0]
+                            if (
+                                len(feature_names.statements) == 1
+                                and name.platformID == 3
+                                and name.platEncID == 1
+                                and name.langID == 0x409
+                            ):
+                                name_text = f"Name: {name.string}"
+                            else:
+                                name_text = str(feature_names)
+                            notes_text = name_text + "\n" + notes_text
+                            notes = True
+                            contents.pop(i)
+                        elif self._font.format_version == 3:
+                            labels = []
+                            for name in feature_names.statements:
+                                if name.platformID == 3 and name.platEncID == 1:
+                                    language = _to_glyphs_language(name.langID)
+                                    labels.append(
+                                        dict(language=language, value=name.string)
+                                    )
+                            if len(labels) == len(feature_names.statements):
+                                feature.labels = labels
+                                contents.pop(i)
+                    break
         if notes:
             feature.notes = notes_text
         if disabled:
