@@ -22,11 +22,13 @@ from .constants import (
     PUBLIC_PREFIX,
     UFO2FT_FEATURE_WRITERS_KEY,
     DEFAULT_FEATURE_WRITERS,
+    DEFAULT_LAYER_NAME,
 )
 
 UFO_DATA_KEY = GLYPHLIB_PREFIX + "ufoData"
 FONT_USER_DATA_KEY = GLYPHLIB_PREFIX + "fontUserData"
 LAYER_LIB_KEY = GLYPHLIB_PREFIX + "layerLib"
+LAYER_NAME_KEY = GLYPHLIB_PREFIX + "layerName"
 GLYPH_USER_DATA_KEY = GLYPHLIB_PREFIX + "glyphUserData"
 NODE_USER_DATA_KEY = GLYPHLIB_PREFIX + "nodeUserData"
 
@@ -77,10 +79,24 @@ def to_ufo_glyph_user_data(self, ufo, glyph):
         ufo.lib[key] = dict(glyph.userData)
 
 
-def to_ufo_layer_lib(self, ufo_layer):
+def to_ufo_layer_lib(self, master, ufo, ufo_layer):
     key = LAYER_LIB_KEY + "." + ufo_layer.name
+    # glyphsLib v5.3.2 and previous versions stored the layer lib in
+    # the GSFont useData under a key named after the layer.
+    # When different original UFOs each had a layer with the same layer name,
+    # only the layer lib of the last one was stored and was exported to UFOs
     if key in self.font.userData.keys():
-        ufo_layer.lib = self.font.userData[key]
+        ufo_layer.lib.update(self.font.userData[key])
+    if key in master.userData.keys():
+        ufo_layer.lib.update(master.userData[key])
+        if LAYER_NAME_KEY in ufo_layer.lib:
+            layer_name = ufo_layer.lib.pop(LAYER_NAME_KEY)
+            # ufoLib2
+            if hasattr(ufo, "renameLayer") and callable(ufo.renameLayer):
+                ufo.renameLayer(ufo_layer.name, layer_name)
+            # defcon
+            else:
+                ufo_layer.name = layer_name
 
 
 def to_ufo_layer_user_data(self, ufo_glyph, layer):
@@ -146,15 +162,24 @@ def to_glyphs_glyph_user_data(self, ufo, glyph):
         glyph.userData = ufo.lib[key]
 
 
-def to_glyphs_layer_lib(self, ufo_layer):
+def to_glyphs_layer_lib(self, ufo_layer, master):
     user_data = {}
     for key, value in ufo_layer.lib.items():
         if _user_data_has_no_special_meaning(key):
             user_data[key] = value
 
+    # the default layer may have a custom name
+    layer_name = ufo_layer.name
+    if (
+        ufo_layer is self._sources[master.id].font.layers.defaultLayer
+        and layer_name != DEFAULT_LAYER_NAME
+    ):
+        user_data[LAYER_NAME_KEY] = ufo_layer.name
+        layer_name = DEFAULT_LAYER_NAME
+
     if user_data:
-        key = LAYER_LIB_KEY + "." + ufo_layer.name
-        self.font.userData[key] = user_data
+        key = LAYER_LIB_KEY + "." + layer_name
+        master.userData[key] = user_data
 
 
 def to_glyphs_layer_user_data(self, ufo_glyph, layer):
