@@ -50,12 +50,15 @@ class GlyphData:
     dictionaries.
     """
 
-    __slots__ = ["names", "alternative_names", "production_names"]
+    __slots__ = ["names", "alternative_names", "production_names", "unicodes"]
 
-    def __init__(self, name_mapping, alt_name_mapping, production_name_mapping):
+    def __init__(
+        self, name_mapping, alt_name_mapping, production_name_mapping, unicodes_mapping
+    ):
         self.names = name_mapping
         self.alternative_names = alt_name_mapping
         self.production_names = production_name_mapping
+        self.unicodes = unicodes_mapping
 
     @classmethod
     def from_files(cls, *glyphdata_files):
@@ -63,6 +66,7 @@ class GlyphData:
         name_mapping = {}
         alt_name_mapping = {}
         production_name_mapping = {}
+        unicodes_mapping = {}
 
         for glyphdata_file in glyphdata_files:
             glyph_data = xml.etree.ElementTree.parse(glyphdata_file).getroot()
@@ -70,6 +74,7 @@ class GlyphData:
                 glyph_name = glyph.attrib["name"]
                 glyph_name_alternatives = glyph.attrib.get("altNames")
                 glyph_name_production = glyph.attrib.get("production")
+                glyph_unicode = glyph.attrib.get("unicode")
 
                 name_mapping[glyph_name] = glyph.attrib
                 if glyph_name_alternatives:
@@ -78,37 +83,43 @@ class GlyphData:
                         alt_name_mapping[glyph_name_alternative] = glyph.attrib
                 if glyph_name_production:
                     production_name_mapping[glyph_name_production] = glyph.attrib
+                if glyph_unicode:
+                    unicodes_mapping[glyph_unicode] = glyph.attrib
 
-        return cls(name_mapping, alt_name_mapping, production_name_mapping)
+        return cls(
+            name_mapping, alt_name_mapping, production_name_mapping, unicodes_mapping
+        )
 
 
-def get_glyph(glyph_name, data=None):
+def get_glyph(glyph_name, data=None, unicodes=None):
     """Return a named tuple (Glyph) containing information derived from a glyph
     name akin to GSGlyphInfo.
 
     The information is derived from an included copy of GlyphData.xml
-    and GlyphData_Ideographs.xml, going purely by the glyph name.
+    and GlyphData_Ideographs.xml, going by the glyph name or unicode fallback.
     """
 
     # Read data on first use.
     if data is None:
         global GLYPHDATA
         if GLYPHDATA is None:
-            try:
-                from importlib.resources import open_binary
-            except ImportError:
-                # use backport for python < 3.7
-                from importlib_resources import open_binary
+            from importlib.resources import open_binary
 
-            GLYPHDATA = GlyphData.from_files(
-                open_binary("glyphsLib.data", "GlyphData.xml"),
-                open_binary("glyphsLib.data", "GlyphData_Ideographs.xml"),
-            )
+            with open_binary("glyphsLib.data", "GlyphData.xml") as f1:
+                with open_binary("glyphsLib.data", "GlyphData_Ideographs.xml") as f2:
+                    GLYPHDATA = GlyphData.from_files(f1, f2)
         data = GLYPHDATA
 
     # Look up data by full glyph name first.
     attributes = _lookup_attributes(glyph_name, data)
 
+    # Look up by unicode
+    if attributes == {} and unicodes is not None:
+        for unicode in unicodes:
+            attributes = _lookup_attributes_by_unicode(unicode, data)
+            if attributes:
+                break
+
     production_name = attributes.get("production")
     if production_name is None:
         production_name = _construct_production_name(glyph_name, data=data)
@@ -151,6 +162,14 @@ def _lookup_attributes(glyph_name, data):
     return attributes
 
 
+def _lookup_attributes_by_unicode(unicode, data):
+    """Look up glyph attributes in data by unicode
+    or return empty dictionary.
+    """
+    attributes = data.unicodes.get(unicode) or {}
+    return attributes
+
+
 def _agl_compliant_name(glyph_name):
     """Return an AGL-compliant name string or None if we can't make one."""
     MAX_GLYPH_NAME_LENGTH = 63
