@@ -62,7 +62,7 @@ class ValueType:
         """Return a typed value representing the structured glyphs strings."""
         raise NotImplementedError("%s read" % type(self).__name__)
 
-    def plistValue(self):
+    def plistValue(self, format_version=2):
         """Return structured glyphs strings representing the typed value."""
         raise NotImplementedError("%s write" % type(self).__name__)
 
@@ -86,7 +86,7 @@ def Vector(dim):
 
         dimension = dim
         default = [0.0] * dimension
-        regex = re.compile("{%s}" % ", ".join(["([-.e\\d]+)"] * dimension))
+        regex = re.compile("[({]%s[})]" % ", ".join(["([-.e\\d]+)"] * dimension))
 
         def fromString(self, src):
             if isinstance(src, list):
@@ -95,9 +95,12 @@ def Vector(dim):
             src = src.replace('"', "")
             return [parse_float_or_int(i) for i in self.regex.match(src).groups()]
 
-        def plistValue(self):
+        def plistValue(self, format_version=2):
             assert isinstance(self.value, list) and len(self.value) == self.dimension
-            return '"{%s}"' % (", ".join(floatToString3(v) for v in self.value))
+            if format_version == 2:
+                return '"{%s}"' % (", ".join(floatToString3(v) for v in self.value))
+            else:
+                return "(%s)" % (",".join(floatToString3(v) for v in self.value))
 
         def __getitem__(self, key):
             assert isinstance(self.value, list) and len(self.value) == self.dimension
@@ -200,7 +203,7 @@ class Rect(Vector(4)):
             value = [value[0], value[1], value2[0], value2[1]]
         super().__init__(value)
 
-    def plistValue(self):
+    def plistValue(self, format_version=2):
         assert isinstance(self.value, list) and len(self.value) == self.dimension
         return '"{{%s, %s}, {%s, %s}}"' % tuple(floatToString3(v) for v in self.value)
 
@@ -245,7 +248,7 @@ class Transform(Vector(6)):
     def __repr__(self):
         return "<affine transformation %s>" % (" ".join(map(str, self.value)))
 
-    def plistValue(self):
+    def plistValue(self, format_version=2):
         assert isinstance(self.value, list) and len(self.value) == self.dimension
         return '"{%s}"' % (", ".join(floatToString5(v) for v in self.value))
 
@@ -284,7 +287,7 @@ class Datetime(ValueType):
     def fromString(self, src):
         return parse_datetime(src)
 
-    def plistValue(self):
+    def plistValue(self, format_version=2):
         return '"%s +0000"' % self.value
 
     def strftime(self, val):
@@ -330,7 +333,7 @@ class Color(ValueType):
     def __repr__(self):
         return self.value.__repr__()
 
-    def plistValue(self):
+    def plistValue(self, format_version=2):
         return str(self.value)
 
 
@@ -380,12 +383,17 @@ class UnicodesList(list):
             unicodes = [str(v) for v in value]
         super().__init__(unicodes)
 
-    def plistValue(self):
+    def plistValue(self, format_version=2):
         if not self:
             return None
         if len(self) == 1:
+            if format_version == 3:
+                return str(int(self[0], 16))
             return self[0]
-        return '"%s"' % ",".join(self)
+        if format_version == 2:
+            return '"%s"' % ",".join(self)
+        else:
+            return "(%s)" % ",".join([str(int(x, 16)) for x in self])
 
 
 class BinaryData(bytes):
@@ -393,7 +401,7 @@ class BinaryData(bytes):
     def fromHex(cls, data):
         return cls(binascii.unhexlify(data))
 
-    def plistValue(self):
+    def plistValue(self, format_version=2):
         # TODO write hex bytes in chunks and split over multiple lines
         # for better readability, like the fonttools xmlWriter does
         return "<%s>" % binascii.hexlify(self).decode()
