@@ -437,6 +437,29 @@ class SetCustomParamsTestBase(object):
         ufo_rt = glyphsLib.to_ufos(font_rt, ufo_module=self.ufo_module)[0]
         self.assertEqual(ufo_rt.lib[UFO2FT_FILTERS_KEY], ufo_filters)
 
+    def test_color_palettes(self):
+        glyphs_palettes = [
+            ["68,0,59,255", "220,187,72,255", "42,255", "87,255", "0,138,255,255"]
+        ]
+        ufo_palettes = [
+            [
+                (0.26666666666666666, 0.0, 0.23137254901960785, 1.0),
+                (0.8627450980392157, 0.7333333333333333, 0.2823529411764706, 1.0),
+                (0.16470588235294117, 0.16470588235294117, 0.16470588235294117, 1.0),
+                (0.3411764705882353, 0.3411764705882353, 0.3411764705882353, 1.0),
+                (0.0, 0.5411764705882353, 1.0, 1.0),
+            ]
+        ]
+        self.font.customParameters["Color Palettes"] = glyphs_palettes
+        self.set_custom_params()
+        self.assertEqual(
+            self.ufo.lib["com.github.googlei18n.ufo2ft.colorPalettes"], ufo_palettes
+        )
+
+        # Test the round-tripping
+        font = glyphsLib.to_glyphs([self.ufo])
+        self.assertEqual(font.customParameters["Color Palettes"], glyphs_palettes)
+
 
 class SetCustomParamsTestUfoLib2(SetCustomParamsTestBase, unittest.TestCase):
     ufo_module = ufoLib2
