@@ -157,10 +157,11 @@ def test_designspace_generation_same_weight_name(tmpdir, ufo_module):
     assert designspace.sources[0].filename != designspace.sources[2].filename
 
 
-def test_designspace_generation_brace_layers(datadir, ufo_module):
-    with open(str(datadir.join("BraceTestFont.glyphs"))) as f:
+@pytest.mark.parametrize("filename", ["BraceTestFont.glyphs", "BraceTestFontV3.glyphs"])
+def test_designspace_generation_brace_layers(datadir, filename, ufo_module):
+    with open(str(datadir.join(filename))) as f:
         font = glyphsLib.load(f)
-    designspace = to_designspace(font, ufo_module=ufo_module)
+    designspace = to_designspace(font, ufo_module=ufo_module, minimal=True)
 
     axes_order = [
         (a.name, a.minimum, a.default, a.maximum, a.map) for a in designspace.axes
@@ -191,9 +192,13 @@ def test_designspace_generation_brace_layers(datadir, ufo_module):
             assert masters[source.filename] is source.font
         masters[source.filename] = source.font
 
+    # Check that brace layer glyph is created
+    assert len(designspace.sources[0].font.layers) == 2
 
-def test_designspace_generation_instances(datadir, ufo_module):
-    with open(str(datadir.join("BraceTestFont.glyphs"))) as f:
+
+@pytest.mark.parametrize("filename", ["BraceTestFont.glyphs", "BraceTestFontV3.glyphs"])
+def test_designspace_generation_instances(datadir, filename, ufo_module):
+    with open(str(datadir.join(filename))) as f:
         font = glyphsLib.load(f)
     designspace = to_designspace(font, ufo_module=ufo_module)
 
@@ -211,8 +216,9 @@ def test_designspace_generation_instances(datadir, ufo_module):
     ]
 
 
-def test_designspace_generation_on_disk(datadir, tmpdir, ufo_module):
-    glyphsLib.build_masters(str(datadir.join("BraceTestFont.glyphs")), str(tmpdir))
+@pytest.mark.parametrize("filename", ["BraceTestFont.glyphs", "BraceTestFontV3.glyphs"])
+def test_designspace_generation_on_disk(datadir, tmpdir, filename, ufo_module):
+    glyphsLib.build_masters(str(datadir.join(filename)), str(tmpdir))
 
     ufo_paths = list(tmpdir.visit(fil="*.ufo"))
     assert len(ufo_paths) == 4  # Source layers should not be written to disk.
@@ -423,7 +429,7 @@ def test_designspace_generation_bracket_no_export_glyph(datadir, ufo_module):
     font.glyphs["E"].export = False
 
     designspace = to_designspace(
-        font, write_skipexportglyphs=True, ufo_module=ufo_module
+        font, write_skipexportglyphs=True, ufo_module=ufo_module, minimal=False
     )
 
     assert "E" in designspace.lib.get("public.skipExportGlyphs")
@@ -492,3 +498,13 @@ def test_designspace_generation_bracket_GDEF(datadir, ufo_module):
             "x.BRACKET.300",
             "x.BRACKET.600",
         }
+
+
+def test_designspace_generation_bracket_glyphs3_simple(datadir, ufo_module):
+    with open(str(datadir.join("Alternate-g3-axis1.glyphs"))) as f:
+        font = glyphsLib.load(f)
+
+    designspace = to_designspace(font, ufo_module=ufo_module)
+
+    for source in designspace.sources:
+        assert "A.BRACKET.600" in source.font
